#!/bin/sh
#\
exec tclsh "$0" "$@"

# Written by Falk Richter
#
# SetUserId.tcl - set the user id bit for cdda2wav
#
# This software stands under the GNU GENERAL PUBLIC LICENSE and comes with 
# NO warranty of any kind. Use at your own risk.
# Report bugs or suggestions to: please see contact in the about-box
# ===============================================================================

proc Main {} \
{
  global env
  
  set StartDir [pwd]
 
  package require msgcat
 
  # we check if a language is set from the operation system
  if {![info exist env(LANG)]} \
  {
    set env(LANG) "en"
  }
  
  msgcat::mclocale $env(LANG)
  msgcat::mcload [file join $StartDir msgs]
  
  # now we can start
  set ReturnValue [CheckLogIn]
  
  if {$ReturnValue == 0} \
  {
    set ReturnValue [WritePermissions]
  }
  
  return $ReturnValue
}


proc CheckLogIn {} \
{
  set UserId ""
 
  set CatchReturn [catch {exec whoami} UserId]
 
  if {$CatchReturn} \
  {
    puts [msgcat::mc "I can't find the program \"whoami\". I need this for the check of the superuser privileges"] 
    return 1
  }
 
  if {$UserId ne "root"} \
  {
    puts [msgcat::mc "You are logged in as: %s. Please start SetUserId.tcl with superuser-privileges." $UserId]
    return 1
  }
  
  return 0
}


proc WritePermissions {} \
{
  set Cdda2WavPath [auto_execok cdda2wav]
  
  if {$Cdda2WavPath eq ""} \
  {
    puts [msgcat::mc "I can't find the program \"cdda2wav\". Please install it first."]
    return 1
  }
  
  set Permissions [file attributes $Cdda2WavPath -permissions]
   
  if {[regexp {[0-9]+4[0-9]+[0-9]+[0-9]+} $Permissions]} \
  {
    puts [msgcat::mc "The permissions are already set right."]
    return 1   
  }
  
  set CatchReturn [catch {file attributes $Cdda2WavPath -permissions 04555} CatchOutput]
  
  if {$CatchReturn} \
  {
    puts [msgcat::mc "I couldn't set the permissions. The reason was: %s" $CatchOutput]
    return 1
  }
  
  puts [msgcat::mc "Permissions successful set!"]
  
  return 0
}


Main