#---multiframe
#
# a main frame which can load different other content-frames
# 
# Copyright (C) 2009 Falk Richter
#
# ::FR_multiFrame::Create <Path>
# creates the main frame
#
# ::FR_multiFrame::NewPage {Path Name {Text ""}}
# creates one content frame
# Arguments:
# <Path> path of the main frame
# <Name> name/path for the content frame
# <Text> name for the labelframe
#
# ::FR_multiFrame::DisplayPage {Path Name}
# shows the content file
# Arguments:
# <Path> path of the main frame
# <Name> name/path for the content frame
#
# ::FR_multiFrame::Size {Path Name}
# calculates the size of the main frame for the biggest content frame
# Arguments:
# <Path> path of the main frame
# <Name> name/path of the biggest content frame


package provide FR_multiFrame 0.1

namespace eval ::FR_multiFrame \
{
  namespace export Create NewPage DisplayPage Size
}


#---creates the main frame
proc ::FR_multiFrame::Create {Path} \
{
  variable VAR
 
  frame .$Path -relief flat
  pack propagate .$Path 0

  set VAR(${Path}_Current) ""
} 


#---creates a new content frame
proc ::FR_multiFrame::NewPage {Path Name {Text ""}} \
{
  variable VAR

  set Page ""
  append Page ${Path} "." ${Name}
 
  labelframe .$Page -text $Text -foreground blue -relief groove -padx 5 -pady 5

  return $Page
}


#---shows the content file
proc ::FR_multiFrame::DisplayPage {Path Name} \
{
  variable VAR
 
  set Page ""
  append Page ${Path} "." ${Name}
 
  if {$VAR(${Path}_Current) ne ""} \
  {
    pack forget .$VAR(${Path}_Current)
  }

  pack .$Page -expand yes -fill both
 
  set VAR(${Path}_Current) $Page
}


#---calculates the size of the main frame for the biggest content frame
proc ::FR_multiFrame::Size {Path Name} \
{
  variable VAR
 
  set Page ""
  append Page ${Path} "." ${Name}
 
  update idletasks
 
  set Width [winfo reqwidth .$Page]
  set Height [winfo reqheight .$Page]
  set BorderWidth [.$Path cget -borderwidth]
 
  set Width [expr {$Width + 2 * $BorderWidth}]
  set Height [expr {$Height + 2 * $BorderWidth}]
 
  .$Path configure -width $Width -height $Height
}
