#---the configuration / options window

proc ::tk_cdconverter::StartConfig {} \
{
  variable CONFIG; variable VAR
 
  set Path [::FR_emptyBox::Create -window .tk_cdconv_config -title [msgcat::mc "Configuration"]]
 
  labelframe .$Path.frame1 -relief groove -text [msgcat::mc "Options"] -foreground blue -padx 5 -pady 5
  frame .$Path.frame2 -borderwidth 1 -relief flat

  pack .$Path.frame1 -side top -expand yes -fill both -padx 5 -pady 5
  pack .$Path.frame2 -side top -fill x


  # We set a variable for each frame - so we can change the order of the frames later easier
  set MainFrame "$Path.frame1"
  set BottomFrame "$Path.frame2"

  # the balloonhelp button
  checkbutton .$MainFrame.balloonhelp -text [msgcat::mc "Show balloon help"] -variable ::tk_cdconverter::CONFIG(BalloonHelp) -pady 10 -justify left -command {::tk_cdconverter::LibBalloonHelpStatus}
 
  # the umlaut button
  checkbutton .$MainFrame.umlaut -text [msgcat::mc "Convert the German umlauts"] -variable ::tk_cdconverter::CONFIG(ReplaceUmlaut) -pady 10 -justify left
 
  # the no-id3v2 button
  checkbutton .$MainFrame.useid3v2 -text [msgcat::mc "Use Id3v2-version for tagging mp3-files"] -variable ::tk_cdconverter::CONFIG(UseId3v2) -justify left
  
  # delete the *.inf-files after tagging
  checkbutton .$MainFrame.delinffiles -text [msgcat::mc "Delete the files with the cd infos after tagging"] -variable ::tk_cdconverter::CONFIG(DeleteInfFiles) -pady 10 -justify left
  
  # the proxy settings
  checkbutton .$MainFrame.proxy -text [msgcat::mc "Use a proxy server"] -variable ::tk_cdconverter::CONFIG(Proxy) -justify left -command [list ::tk_cdconverter::ProxyState $MainFrame] 
  
  label .$MainFrame.proxyhost_comment -text [msgcat::mc "Proxyserver:"]
  entry .$MainFrame.proxyhost
  label .$MainFrame.proxyport_comment -text [msgcat::mc "Proxyport:"]
  entry .$MainFrame.proxyport
 
  .$MainFrame.proxyhost insert end $CONFIG(ProxyHost)
  .$MainFrame.proxyport insert end $CONFIG(ProxyPort)
  
  ::tk_cdconverter::ProxyState $MainFrame
 
  set row 0
  ::FR_general::Topic $MainFrame $row [msgcat::mc "General settings:"] 3 
  eval ::FR_general::PackHelp_1 [incr row] .$MainFrame.balloonhelp 0 w
  ::FR_general::Line $MainFrame [incr row] 3
 
  ::FR_general::Topic  $MainFrame [incr row] [msgcat::mc "Settings for the tag-tool:"] 3
  eval ::FR_general::PackHelp_1 [incr row] .$MainFrame.umlaut 0 w
  eval ::FR_general::PackHelp_1 [incr row] .$MainFrame.useid3v2 0 w
  eval ::FR_general::PackHelp_1 [incr row] .$MainFrame.delinffiles 0 w
  ::FR_general::Line $MainFrame [incr row] 3
 
  ::FR_general::Topic $MainFrame [incr row] [msgcat::mc "If you need a proxy-server for internet access:"] 3 
  eval ::FR_general::PackHelp_1 [incr row] .$MainFrame.proxy 0 w
  eval ::FR_general::PackHelp_1 [incr row] .$MainFrame.proxyhost_comment 0 w .$MainFrame.proxyhost 2 ew 
  eval ::FR_general::PackHelp_1 [incr row] .$MainFrame.proxyport_comment 0 w .$MainFrame.proxyport 2 ew
  
  grid columnconfigure .$MainFrame 1 -weight 1
    
  set ButtonText [::FR_general::ButtonsWithSameWidth [list [msgcat::mc "Cancel"] [msgcat::mc "Save"]]]
 
  button .$BottomFrame.cancel -text [lindex $ButtonText 0] -width [lindex $ButtonText 2] -state normal -command [list ::tk_cdconverter::ConfigCancel $Path]

  button .$BottomFrame.okay -text [lindex $ButtonText 1] -state normal -width [lindex $ButtonText 2] -command [list ::tk_cdconverter::ConfigOkay $Path $MainFrame]
  	       
  pack .$BottomFrame.okay -side left -padx {5 0} -pady 5 -expand yes -fill x
  pack .$BottomFrame.cancel -side left -padx 5 -pady 5 -expand yes -fill x
  
  bind .$Path <KeyPress-Escape> [list .$BottomFrame.cancel invoke]
  bind .$Path <KeyPress-Return> [list .$BottomFrame.okay invoke]
 
  ::FR_balloonHelp::AddHelp ".$MainFrame.umlaut" [msgcat::mc "Not all Player can show the German umlauts, in this case a conversation is useful"]
  ::FR_balloonHelp::AddHelp ".$MainFrame.useid3v2" [msgcat::mc "Not all Player can show mp3-tags in the id3v2 format. If you deactivate this we are using the older id3v1 version."]
  ::FR_balloonHelp::AddHelp ".$MainFrame.delinffiles" [msgcat::mc "During ripping the program has generated an info-file for each song which contains all informations about the cd. This informations are now written down to your compressed files, so normaly you don't need this files anymore."]
  
  ::tk_cdconverter::LibBalloonHelpStatus

  ::FR_general::CenterWindow .$Path $VAR(Parent)
  wm deiconify .$Path
 
  wm protocol .$Path WM_DELETE_WINDOW [list .$BottomFrame.cancel invoke]
}


#---activate or deactivate the proxyhost and proxyport
proc ::tk_cdconverter::ProxyState {Path} \
{
  variable CONFIG
 
  if {$CONFIG(Proxy)} \
  {
    .$Path.proxyhost configure -state normal
    .$Path.proxyport configure -state normal
  } \
  else \
  {
    .$Path.proxyhost configure -state disable
    .$Path.proxyport configure -state disable
  }
}


#---the cancel button
proc ::tk_cdconverter::ConfigCancel {Path} \
{
  variable VAR
  
  focus $VAR(Parent)
  destroy .$Path
}

 
#---the okay button
proc ::tk_cdconverter::ConfigOkay {Path MainFrame} \
{
  variable CONFIG; variable VAR
  
  if {$CONFIG(Proxy)} \
  {
     set CONFIG(ProxyHost) [.$MainFrame.proxyhost get]
     set CONFIG(ProxyPort) [.$MainFrame.proxyport get]
  }
    
  # we write the config-file
  ::FR_config::SaveConfig $VAR(NameSpace)
  
  # we close the configuration dialog
  ::tk_cdconverter::ConfigCancel $Path
}
