# tagger.tcl

#---the main proc for the converter
proc ::tk_cdconverter::TaggerMain {} \
{
  variable VAR
  
  # we generate the list for the genre combobox 
  if {![info exists VAR(GenreList)]} \
  {
    ::tk_cdconverter::LibGenerateGenreList
  }
  
  # the count of the (later generated) title fields
  set VAR(TaggerTitleFields) 0
  
  # the list with all files of the chosen directory
  set VAR(AllFiles) ""
  
  ::tk_cdconverter::TaggerGui
}


#---after a switch to this program part we make some presettings
proc ::tk_cdconverter::TaggerClearGui {} \
{
  variable VAR
   
  .$VAR(CommonTagsFrame).artist delete 0 end
  .$VAR(CommonTagsFrame).album delete 0 end
  .$VAR(CommonTagsFrame).year delete 0 end
  
  .$VAR(CommonTagsFrame).year insert end [clock format [clock seconds] -format %Y]
  
  # we destroy all existing widgets in the scrollframe
  if {$VAR(TaggerTitleFields) != 0} \
  {
    foreach Arg [winfo children .$VAR(TackerTitleFrame)] \
    {
      catch {destroy $Arg}
    }
    
    set VAR(TaggerTitleFields) 0
  }
  
  ::FR_general::MakeDisable .$VAR(taggerframe).start .$VAR(taggerframe).tools
}


#---we create and place all gui elements
proc ::tk_cdconverter::TaggerGui {} \
{
  variable VAR; variable CONFIG
  
  # the widgets
  label .$VAR(taggerframe).format_comment -pady 10 -text [msgcat::mc "Set tags for:"]

  ::FR_comboBox::cb .$VAR(taggerframe).format -variable ::tk_cdconverter::CONFIG(TaggerFormat)
  ::FR_comboBox::cb .$VAR(taggerframe).format -listbox_content $VAR(TagToolList) -entry_width auto -default_value $CONFIG(TaggerFormat) -command {::tk_cdconverter::TitleFieldsCreate}

  label .$VAR(taggerframe).outputdir_comment  -pady 10 -text [msgcat::mc "Choose the directory with your compressed files:"]
  button .$VAR(taggerframe).outputdir -text [msgcat::mc "Open"] -command {::tk_cdconverter::TaggerOutputDir}
    
  label .$VAR(taggerframe).tools_comment -pady 10 -text [msgcat::mc "Tools which help you to set the tags:"]
  button .$VAR(taggerframe).tools -text [msgcat::mc "Choose"] -command {tk_popup .$::tk_cdconverter::VAR(taggerframe).tools_menu [winfo pointerx .] [winfo pointery .]}

  menu .$VAR(taggerframe).tools_menu -tearoff 0
  .$VAR(taggerframe).tools_menu add command -label [msgcat::mc "Use the infos which are saved on the CD"] -command {::tk_cdconverter::ReadCdText}
  .$VAR(taggerframe).tools_menu add separator
  .$VAR(taggerframe).tools_menu add command -label [msgcat::mc "Get the infos from the internet freedb-database"] -command {::tk_cdconverter::ReadFreeDb}
  
  set VAR(CommonTagsFrame) "$VAR(taggerframe).commontags"
  frame .$VAR(CommonTagsFrame) -relief flat
  
  set VAR(TitleTagsFrame) "$VAR(taggerframe).titletags"
  frame .$VAR(TitleTagsFrame) -relief flat -pady 5
  
  button .$VAR(taggerframe).start -text [msgcat::mc "Start"] -state disabled -command {::tk_cdconverter::TaggerStartButton}
  
  # the placement
  set row 0
  eval ::FR_general::PackHelp_1 $row .$VAR(taggerframe).format_comment 0 w .$VAR(taggerframe).format 2 ew
  ::FR_general::Line $VAR(taggerframe) [incr row] 3
 
  eval ::FR_general::PackHelp_1 [incr row] .$VAR(taggerframe).outputdir_comment 0 w .$VAR(taggerframe).outputdir 2 ew
  ::FR_general::Line $VAR(taggerframe) [incr row] 3
  
  eval ::FR_general::PackHelp_1 [incr row] .$VAR(taggerframe).tools_comment 0 w .$VAR(taggerframe).tools 2 ew
  ::FR_general::Line $VAR(taggerframe) [incr row] 3
  
  eval ::FR_general::PackHelp_2 [incr row] .$VAR(CommonTagsFrame) 0 3 nsew
  
  set MainRow [incr row]
  eval ::FR_general::PackHelp_2 $MainRow .$VAR(TitleTagsFrame) 0 3 nsew
  
  eval ::FR_general::PackHelp_2 [incr row] .$VAR(taggerframe).start 0 3 ew
  
  grid columnconfigure .$VAR(taggerframe) 1 -weight 1
  grid rowconfigure .$VAR(taggerframe) $MainRow -weight 1
  
  # the scroll window
  set VAR(TackerTitleFrame) [::FR_general::ScrollWindow $VAR(TitleTagsFrame)]
  
  # the widgets of the common tags frame
  label .$VAR(CommonTagsFrame).common_comment -pady 10 -text [msgcat::mc "Tags which are used for all songs:"]
  
  label .$VAR(CommonTagsFrame).artist_comment -text [msgcat::mc "Artist:"]
  entry .$VAR(CommonTagsFrame).artist -state normal -width 28

  label .$VAR(CommonTagsFrame).album_comment -text [msgcat::mc "Album:"]
  entry .$VAR(CommonTagsFrame).album -state normal -width 28

  label .$VAR(CommonTagsFrame).year_comment -text [msgcat::mc "Year:"]
  entry .$VAR(CommonTagsFrame).year -state normal -width 4

  .$VAR(CommonTagsFrame).year insert end [clock format [clock seconds] -format %Y]

  label .$VAR(CommonTagsFrame).genre_comment -text [msgcat::mc "Genre:"]
  ::FR_comboBox::cb .$VAR(CommonTagsFrame).genre -variable ::tk_cdconverter::CONFIG(Genre)
  ::FR_comboBox::cb .$VAR(CommonTagsFrame).genre -listbox_content $VAR(GenreListTxt) -entry_width auto -default_value $CONFIG(Genre)
  
  # we put the label for the next frame here
  label .$VAR(CommonTagsFrame).titletags_comment -pady 10 -text [msgcat::mc "Specific Tags for each song:"]
  
  # the placement of the common tags frame
  set row 0
  eval ::FR_general::PackHelp_2 $row .$VAR(CommonTagsFrame).common_comment 0 2 w
  eval ::FR_general::PackHelp_1 [incr row] .$VAR(CommonTagsFrame).artist_comment 0 w .$VAR(CommonTagsFrame).artist 1 ew
  eval ::FR_general::PackHelp_1 [incr row] .$VAR(CommonTagsFrame).album_comment 0 w .$VAR(CommonTagsFrame).album 1 ew
  eval ::FR_general::PackHelp_1 [incr row] .$VAR(CommonTagsFrame).year_comment 0 w .$VAR(CommonTagsFrame).year 1 w
  eval ::FR_general::PackHelp_1 [incr row] .$VAR(CommonTagsFrame).genre_comment 0 w .$VAR(CommonTagsFrame).genre 1 w
  eval ::FR_general::PackHelp_2 [incr row] .$VAR(CommonTagsFrame).titletags_comment 0 2 w
  
  grid columnconfigure .$VAR(CommonTagsFrame) 1 -weight 1
  
  ::FR_balloonHelp::AddHelp ".$VAR(taggerframe).outputdir" [msgcat::mc "This is the directory with your compressed files (e.g. *.mp3, *.ogg...)"]
  ::FR_balloonHelp::AddHelp ".$VAR(taggerframe).tools" [msgcat::mc "To get the informations about titles and artists there are some tools:\n\nUse the information from the cd or from the internet"]
  ::FR_balloonHelp::AddHelp ".$VAR(CommonTagsFrame).artist" [msgcat::mc "If all tracks have the same artist - please put it in here"]
  
  ::tk_cdconverter::LibBalloonHelpStatus
}


#---load the output directory
proc ::tk_cdconverter::TaggerOutputDir {} \
{
  variable CONFIG; variable VAR

  set OutputDir [::FR_dirBox::Create -initialdir $CONFIG(OutputDir) -title [msgcat::mc "Choose the directory with your compressed files:"] -parent .$VAR(taggerframe) -permission "writable" -filetype $VAR(TagToolFileExtension)]

  if {$OutputDir ne ""} \
  {
    cd $OutputDir
    
    set CONFIG(OutputDir) $OutputDir

    ::FR_msgWindow::Add -format Info -msg [msgcat::mc "The directory was set to: %s" $OutputDir]
    
    ::tk_cdconverter::TitleFieldsCreate
  }
}


#---check if all elementes of a list are equal
proc ::tk_cdconverter::TaggerAllEqual {List} \
{
  set SortedList [lsort -unique $List]

  if {[llength $SortedList] == 1} \
  {
    return [lindex $SortedList 0]
  } \
  else \
  {
    return ""
  }
}


#---fill the entry widgets with new contents
proc ::tk_cdconverter::TaggerInsertEntryContent {Widget {Value ""}} \
{
  if {[winfo exists $Widget]} \
  {
    $Widget delete 0 end
    
    if {$Value ne ""} \
    { 
      $Widget insert end $Value
    }
  }
}


#---parse the cd-text information
proc ::tk_cdconverter::ReadCdText {} \
{
  variable VAR; variable CONFIG
  
  if {$VAR(AllFiles) eq ""} \
  {
    ::FR_msgWindow::Add -format Error -msg [msgcat::mc "I didn't find any files in the %s-format. Have you chosen the right directory?" $CONFIG(TaggerFormat)]
    return 1
  }
  
  set Taglist ""
  
  foreach File $VAR(AllFiles) \
  {
    set InfFile [file rootname $File]
    append InfFile ".inf"
    
    if {![file exists $InfFile]} \
    {
      ::FR_msgWindow::Add -format Error -msg [msgcat::mc "We can't find the cd-text file: %s.\n\nSo we can't fetch the track information from the cd." $InfFile]
      return 1
    }
    
    set TagListTmp [::tk_cdconverter::ReadInfContent $InfFile]
    
    if {[lindex $TagListTmp 2] eq ""} \
    {
      ::FR_msgWindow::Add -format Info -msg [msgcat::mc "We didn't find any information for file: %s." $File]
    }
    
    eval lappend TagList $TagListTmp
  }
  
  set AllArtists ""; set AllAlbums ""
  
  set Index 1
  
  # we insert the titles
  foreach "Title Artist Album Track" $TagList \
  {  
    ::tk_cdconverter::TaggerInsertEntryContent .$VAR(TackerTitleFrame).title$Index $Title
    ::tk_cdconverter::TaggerInsertEntryContent .$VAR(TackerTitleFrame).artist$Index $Artist
    
    lappend AllArtists $Artist
    lappend AllAlbums $Album
    
    incr Index
  }
  
  set AllArtists [::tk_cdconverter::TaggerAllEqual $AllArtists]
  
  ::tk_cdconverter::TaggerInsertEntryContent .$VAR(CommonTagsFrame).artist $AllArtists
  
  set AllAlbums [::tk_cdconverter::TaggerAllEqual $AllAlbums]
  
  ::tk_cdconverter::TaggerInsertEntryContent .$VAR(CommonTagsFrame).album $AllAlbums
}


#---read the inf-files generated by cdda2wav
proc ::tk_cdconverter::ReadInfContent {File} \
{ 
  set Album ""; set Artist ""; set Title ""; set Track 0
 
  if {[file exists $File] && [file readable $File]} \
  {
    set Fid [open $File r]
    set InfContent [read $Fid]
    close $Fid
  } \
  else \
  {
    return [list $Title $Artist $Album $Track]
  }
 
  # usually we must find the track number also if a cd text isn't available
  regexp {Tracknumber=([\t]+)([0-9]+)} $InfContent All Tab Track
 
  # wasn't found a album title we assume there isn't a cd text on this cd and return the empty list
  if {[regexp {Albumtitle=([\t]+)'([^']+)'} $InfContent All Tab Album] == 0} \
  {
    return [list $Title $Artist $Album $Track]
  }
 
  regexp {Performer=([\t]+)'([^']+)'} $InfContent All Tab Artist
  regexp {Tracktitle=([\t]+)'([^']+)'} $InfContent All Tab Title
 
  return [list $Title $Artist $Album $Track]
}
 

#---read from the inf-files the information which are needed for the freedb access 
proc ::tk_cdconverter::ReadInfFreeDbPart {File} \
{ 
  set DiscId 0; set Track 0; set CountOfTracks 0; set SectorList ""; set CdTime 0
 
  # if we can't open the inf file we will return an empty list
  if {[file exists $File] && [file readable $File]} \
  {
    set Fid [open $File r]
    set InfContent [read $Fid]
    close $Fid
  } \
  else \
  {
    return [list $DiscId $Track $CountOfTracks $CdTime $SectorList]
  }
  
  # this information were created by cdda2wav 
  regexp {CDDB_DISCID=([\t]+)([^\n]+)} $InfContent All Tab DiscId
  
  regexp {Tracknumber=([\t]+)([0-9]+)} $InfContent All Tab Track
  
  # this information were added from cdripper.tcl
  regexp {Tracks:([\s]+)([0-9]+)} $InfContent All Space CountOfTracks
  regexp {Cd-Time:([\s]+)([0-9]+)} $InfContent All Space CdTime
  regexp {Sectors:([\s]+)([^\n]+)} $InfContent All Space SectorList
  
  return [list $DiscId $Track $CountOfTracks $CdTime $SectorList]
}


#---the start button
proc ::tk_cdconverter::TaggerStartButton {} \
{
  variable CONFIG
  
  ::tk_cdconverter::TaggerWriteInfo
   
  # we read the compressed files again to control the write sucess
  ::tk_cdconverter::TitleFieldsCreate
  
  if {$CONFIG(DeleteInfFiles)} \
  {
    ::tk_cdconverter::DeleteInfFiles
  }

  ::FR_messageBox::Create -message [msgcat::mc "We have written and re-read all tags.\nYou can now control the correctness."] -title [msgcat::mc "End of job"] -type Info
}


#---generate one line per song and fill the entry-widgets with the title/artist informations
proc ::tk_cdconverter::TitleFieldsCreate {} \
{
  variable CONFIG; variable VAR
  
  # we destroy all existing widgets in the scrollframe
  if {$VAR(TaggerTitleFields) != 0} \
  {
    foreach Arg [winfo children .$VAR(TackerTitleFrame)] \
    {
      catch {destroy $Arg}
    }
    
    set VAR(TaggerTitleFields) 0
  }
  
  set VAR(AllFiles) [::tk_cdconverter::LibCheckDirContent $CONFIG(OutputDir) $CONFIG(TaggerFormat)]

  if {$VAR(AllFiles) != ""} \
  {
    set AllArtists ""; set AllAlbums ""; set AllGenres ""; set AllYears ""
   
    # the labels in the table head
    label .$VAR(TackerTitleFrame).file_comment -text [msgcat::mc "Filename"]
    label .$VAR(TackerTitleFrame).artist_comment -text [msgcat::mc "Artist"]
    label .$VAR(TackerTitleFrame).title_comment -text [msgcat::mc "Title"]

    set Index 0
    eval ::FR_general::PackHelp_1 $Index .$VAR(TackerTitleFrame).file_comment 0 w .$VAR(TackerTitleFrame).artist_comment 1 w .$VAR(TackerTitleFrame).title_comment 2 w
    
    foreach File $VAR(AllFiles) \
    {
      incr Index
     
      # for a good looking of the showed file name: 1. Song; 10.Song
      set FileName "${Index}."
   
      if {$Index < 10} \
      {
        append FileName " "
      }

      # we limit the count of letters in the file name to 20
      if {[string length $File] > 20} \
      {
        set RootName [file rootname $File]
        set FileExtension [file extension $File]
    
        # range form 0...18 are 19 characters + "~" = 20
        set NameLimit [expr {18 - [string length $FileExtension]}]

        append FileName [string range $RootName 0 $NameLimit] "~" $FileExtension
      } \
      else \
      {
        append FileName $File
      }
   
      label .$VAR(TackerTitleFrame).file$Index -text "$FileName"
      entry .$VAR(TackerTitleFrame).artist$Index -state normal -width 28 -validate focusin -validatecommand "::FR_general::ScrollWindowShowWidget $::tk_cdconverter::VAR(TitleTagsFrame) %W"
      entry .$VAR(TackerTitleFrame).title$Index -state normal -width 28 -validate focusin -validatecommand "::FR_general::ScrollWindowShowWidget $::tk_cdconverter::VAR(TitleTagsFrame) %W"

      eval ::FR_general::PackHelp_1 $Index .$VAR(TackerTitleFrame).file$Index 0 w .$VAR(TackerTitleFrame).artist$Index 1 ew .$VAR(TackerTitleFrame).title$Index 2 ew

      set TagList [::tk_cdconverter::LibReadTag [file join $CONFIG(OutputDir) $File] $CONFIG(TaggerFormat)]
     
      # we generate lists with the informations of all compressed files
      lappend AllArtists [lindex $TagList 1]
      lappend AllYears [lindex $TagList 2]
      lappend AllAlbums [lindex $TagList 3]
      lappend AllGenres [lindex $TagList 4]

      .$VAR(TackerTitleFrame).artist$Index insert end [lindex $TagList 1]
      .$VAR(TackerTitleFrame).title$Index insert end [lindex $TagList 0]
    }

    grid columnconfigure .$VAR(TackerTitleFrame) 1 -weight 1
    grid columnconfigure .$VAR(TackerTitleFrame) 2 -weight 1

    # we need the count of widgets for the destroying on the start of the next run of titleartist
    set VAR(TaggerTitleFields) $Index

    # if all items are equal we will fill in the first element - if not we will fill in "" or "Other" in the genre case
    ::tk_cdconverter::TaggerInsertEntryContent .$VAR(CommonTagsFrame).artist [::tk_cdconverter::TaggerAllEqual $AllArtists]
    ::tk_cdconverter::TaggerInsertEntryContent .$VAR(CommonTagsFrame).album [::tk_cdconverter::TaggerAllEqual $AllAlbums]
    ::tk_cdconverter::TaggerInsertEntryContent .$VAR(CommonTagsFrame).year [::tk_cdconverter::TaggerAllEqual $AllYears]
    
    set CONFIG(Genre) [::tk_cdconverter::TaggerAllEqual $AllGenres]
   
    if {$CONFIG(Genre) == ""} \
    {
      set CONFIG(Genre) "Other"
    }
 
    ::FR_general::ScrollWindowRefresh $VAR(TitleTagsFrame)

    ::FR_general::MakeNormal .$VAR(taggerframe).start .$VAR(taggerframe).tools

    # we configure the scrollincrement with help of the first entry-box
    ::FR_general::ScrollWindowIncrement $VAR(TitleTagsFrame) $VAR(TackerTitleFrame).artist1
  } \
  else \
  {
    ::FR_general::MakeDisable .$VAR(taggerframe).start .$VAR(taggerframe).tools

    .$VAR(CommonTagsFrame).artist delete 0 end
    .$VAR(CommonTagsFrame).album delete 0 end
    .$VAR(CommonTagsFrame).year delete 0 end
    
    .$VAR(CommonTagsFrame).year insert end [clock format [clock seconds] -format %Y]
    
    ::FR_msgWindow::Add -format Error -msg [msgcat::mc "I didn't find any files in the %s-format. Have you chosen the right directory?" $CONFIG(TaggerFormat)]
  }
}


#---we read out the entry-widget and replace the German umlauts
proc ::tk_cdconverter::TaggerCheckContent {Widget} \
{
  variable CONFIG

  set Content [$Widget get]

  if {$Content ne ""} \
  {
    if {$CONFIG(ReplaceUmlaut)} \
    {
      regsub -all -- {} $Content {ue} Content
      regsub -all -- {} $Content {Ue} Content
      regsub -all -- {} $Content {oe} Content
      regsub -all -- {} $Content {Oe} Content
      regsub -all -- {} $Content {ae} Content
      regsub -all -- {} $Content {Ae} Content
      regsub -all -- {} $Content {ss} Content
    }
  }
  
  return $Content
}


#---we read out the content of the entry widget and start the writing
proc ::tk_cdconverter::TaggerWriteInfo {} \
{
  variable CONFIG; variable VAR

  # first we catch all informations from the entry boxes
  set Album [::tk_cdconverter::TaggerCheckContent .$VAR(CommonTagsFrame).album]
  set Year [::tk_cdconverter::TaggerCheckContent .$VAR(CommonTagsFrame).year]
  set Artist [::tk_cdconverter::TaggerCheckContent .$VAR(CommonTagsFrame).artist]

  # is the general artist entry empty we get the artist informations from each title
  # we use this in the next foreach loop
  if {$Artist eq ""} \
  {
    set NoCommonArtist 1
  } \
  else \
  {
    set NoCommonArtist 0
  }

  # we write all fields to the files
  set Index 0
 
  foreach File $VAR(AllFiles) \
  {
    incr Index

    set Title [::tk_cdconverter::TaggerCheckContent .$VAR(TackerTitleFrame).title$Index]
   
    if {$NoCommonArtist} \
    {
      set Artist [::tk_cdconverter::TaggerCheckContent .$VAR(TackerTitleFrame).artist$Index]
    }
    
    ::tk_cdconverter::LibWriteTag $Title $Artist $Year $Album $CONFIG(Genre) $Index [file join $CONFIG(OutputDir) $File] $CONFIG(TaggerFormat)
  }
}


#---get the information from the internet
proc ::tk_cdconverter::ReadFreeDb {} \
{
  variable VAR; variable CONFIG
  
  if {$VAR(AllFiles) eq ""} \
  {
    ::FR_msgWindow::Add -format Error -msg [msgcat::mc "I didn't find any files in the %s-format. Have you chosen the right directory?" $CONFIG(TaggerFormat)]
    return 1
  }
  
  set InfList ""; set TrackWithDiscIdZero 0
  
  foreach File $VAR(AllFiles) \
  {
    set InfFile [file rootname $File]
    append InfFile ".inf"
    
    if {![file exists $InfFile]} \
    {
      ::FR_msgWindow::Add -format Error -msg [msgcat::mc "We can't find the cd-text file: %s. So we can't fetch the track information from the internet." $InfFile]
      return 1
    }
    
    set TagList [::tk_cdconverter::ReadInfFreeDbPart $InfFile]
    
    #elements of TagList: DiscId Track CountOfTracks CdTime SectorList
    
    # if DiscId == 0 (no information available) we save as track number an ongoing index 
    if {[lindex $TagList 0] == 0} \
    {
      ::FR_msgWindow::Add -format Info -msg [msgcat::mc "We didn't find the disc-id in the file: %s." $File]
      incr TrackWithDiscIdZero
      set TagList [lreplace $TagList 1 1 $TrackWithDiscIdZero]
    }
     
    eval lappend InfList $TagList
  }
  
  set LastDiscId ""; set SortedList ""
  
  for {set i 0} {$i < [llength $InfList]} {incr i 5} \
  {
    set DiscId [lindex $InfList $i]
    
    # if there a change in the disc-id we record this
    if {$DiscId ne $LastDiscId} \
    {
      # we add the end index of this disc-id, but not if the SortedList is empty yet
      if {[llength $SortedList] > 0} \
      {
        # we fetch the track number of the last record
        set EndIndex [lindex $InfList [expr {$i - 4}]]
        set SortedList [linsert $SortedList end $EndIndex]
      }
      
      # we save the start index and the disc-id
      eval lappend SortedList $DiscId [lrange $InfList [expr {$i + 1}] [expr {$i + 4}]]
      set LastDiscId $DiscId
    }
  }
  
  # we add the end index for the last disc-id, but not if the SortedList is empty
  if {[llength $SortedList] > 0} \
  {
    set EndIndex [lindex $InfList [expr {$i - 4}]]
    set SortedList [linsert $SortedList end $EndIndex]
  }
  
  # now the InfList has the format: DiscId StartTrack CountOfTracks CdTime SectorList EndTrack
  set InfList $SortedList
  unset SortedList

  set Index 1
  set FillInCommonFields 1
  set VAR(FreeDbCommonFields) ""
  
  foreach "DiscId StartTrack CountOfTracks CdTime SectorList EndTrack" $InfList \
  {      
    if {$DiscId != 0} \
    {
      set VAR(FreeDbTransferFinished) "-1"
      set SectorList [list $SectorList]

      ::tk_cdconverter::FreeDbSendQueryCmd $DiscId $CountOfTracks $CdTime $SectorList
      vwait ::tk_cdconverter::VAR(FreeDbTransferFinished)
      
      # 0...all was successful
      # 1...abort by the user or nothing found
      # 2...hard error
      switch -- $VAR(FreeDbTransferFinished) \
      {
        0 {set Index [::tk_cdconverter::FreeDbFillInResults $Index $StartTrack $EndTrack]} \
        1 {set Index [expr {$Index + ($EndTrack - $StartTrack) + 1}]; set FillInCommonFields 0} \
        2 {set FillInCommonFields 0; break} \
      }
    } \
    else \
    {
      # the DiscId is zero, so a freedb demand doesn't make sense
      set Index [expr {$Index + ($EndTrack - $StartTrack) + 1}]
      # if we don't ask the freedb, we don't can fill out the common fields
      set FillInCommonFields 0
    }
  }
  
  if {$FillInCommonFields} \
  {
    ::tk_cdconverter::FreeDbFillInCommonFields
  }
}


#---we start here with the freedb internet demand
proc ::tk_cdconverter::FreeDbSendQueryCmd {DiscId CountOfTracks CdTime SectorList} \
{
  global env
  variable ABOUT; variable CONFIG; variable VAR 
 
  ::tk_cdconverter::Packageload http
 
  # if a proxy used we set the configuration
  if {$CONFIG(Proxy)} \
  {
    http::config -proxyhost $CONFIG(ProxyHost) -proxyport $CONFIG(ProxyPort)
  }

  # we built the query url (including the handshake)
  set Url ""
  append Url $CONFIG(FreeDbAddress) ? [http::formatQuery cmd "cddb query $DiscId $CountOfTracks [lindex $SectorList 0] $CdTime" hello "$env(USER) $CONFIG(Domain) $ABOUT(Name) $ABOUT(Version)" proto "5"]

  # we fetch the handshake url
  set CatchReturn [catch {http::geturl $Url -command "::tk_cdconverter::FreeDbParseQuery"} CatchContent]
 
  if {$CatchReturn} \
  {
    ::FR_msgWindow::Add -format Error -msg [msgcat::mc "An error occured during connecting to the freedb-database: %s" $CatchContent] 
    set VAR(FreeDbTransferFinished) 2
  }
}


#---after a query we parse the answer from freedb
proc ::tk_cdconverter::FreeDbParseQuery {Token} \
{
  variable VAR  
 
  set Data [http::data $Token]

  # we check the return values (error-code) from the freedb-database (querry)
  if {[regexp {^(200)} $Data]} \
  {
    ::tk_cdconverter::FreeDbSendReadCmd $Data "exact"
  } \
  elseif {[regexp {^(210)} $Data]} \
  {
    ::tk_cdconverter::FreeDbSendReadCmd $Data "multiple"
  } \
  elseif {[regexp {^(211)} $Data]} \
  {
    ::tk_cdconverter::FreeDbSendReadCmd $Data "inexact"
  } \
  elseif {[regexp {^(202)} $Data]} \
  {
    ::FR_msgWindow::Add -format Error -msg [msgcat::mc "Sorry, this cd isn't (yet) in the freedb-database"] 
    set VAR(FreeDbTransferFinished) 1
  } \
  else \
  {
    ::FR_msgWindow::Add -format Error -msg [msgcat::mc "Couldn't get the infos from the freedb-database. The reason was: %s" $Data]
    set VAR(FreeDbTransferFinished) 2
  }
  
  http::cleanup $Token
}


#---this procedure offers a choice window if there are more than one match and starts the reading   
proc ::tk_cdconverter::FreeDbSendReadCmd {Data Matches} \
{  
  global env
  variable ABOUT; variable CONFIG; variable VAR
  
  # we delete the return-code from freedb i.g. 200 Found exact match (until terminating `.')
  if {[regexp {(^[0-9]+)(.*)\(until terminating `.'\)\n} $Data]} \
  {
    set Data [regsub {(^[0-9]+)(.*)\(until terminating `.'\)\n} $Data ""]
  } \
  elseif {[regexp {(^[0-9]+)} $Data]} \
  {
    # we delete the return-code from freedb when it is only a number
    set Data [regsub {(^[0-9]+)} $Data ""]
  }
  
  # we find the lines with genre-discid-artist/title
  set DataList [regexp -all -inline -- {([a-z]+)([\s]+)([^\s]+)([\s]+)([^\n]*)} $Data]
  set DataListLength [llength $DataList]

  
  set VAR(FreeDbButtonActionFinished) 0
  # FreeDbMultipleValues is the index of the chosen match
  set VAR(FreeDbMultipleValues) ""
  
  if {$Matches eq "exact"} \
  {
    # 0 represents the first entry, by a exact match - there only exists one...
    set VAR(FreeDbMultipleValues) 0
  } \
  else \
  {
    # if we have more than one match, we open a window with the choice possibility
   
    # we generate an empty window with ok/chancel-buttons
    set Window [::FR_emptyBox::Create -window .freedb_matchchoice -title [msgcat::mc "Multiple matches"]]

    # the frames
    frame .$Window.textframe -relief flat
    frame .$Window.buttonframe -relief flat
 
    set TextFrame "$Window.textframe"
    set ButtonFrame "$Window.buttonframe"
 
    pack .$TextFrame -side top -expand yes -fill both -padx 5 -pady 5
    pack .$ButtonFrame -side top -fill x -padx 5 -pady 5
 
    # the text widget  
    set TextWidget [::FR_general::TextwidgetWithScrollbars .$TextFrame]
    
    if {$Matches eq "multiple"} \
    {
      .$TextWidget insert end [msgcat::mc "There are multiple entries in the freedb-database. Please choose one of them:"]
    } \
    elseif {$Matches eq "inexact"} \
    {
      .$TextWidget insert end [msgcat::mc "I only found inexact entries in the freedb-database. Please choose one of them or press chancel for nothing:"]
    }
   
    .$TextWidget insert end "\n\n"
    
    # we generate the checkbuttons for the choice
    set CheckBoxList ""

    for {set i 1} {$i < [llength $DataList]} {incr i 6} \
    {
      lappend CheckBoxList [msgcat::mc "%s (categorie: %s)" [lindex $DataList [expr {$i + 4}]] [lindex $DataList $i]]
    }
   
    set CheckBoxListLastIndex [expr {[llength $CheckBoxList] - 1}]
   
    # Index is index and on-value for the checkbutton
    set Index 0
   
    # we set the checkbutton-variable at the first match
    set VAR(FreeDbMultipleValues) 0
   
    # we generate for each freedb-match one checkbutton
    foreach Arg $CheckBoxList \
    {
      checkbutton .$TextWidget.cb$Index -text $Arg -background white -activebackground white -highlightbackground white -variable ::tk_cdconverter::VAR(FreeDbMultipleValues) -onvalue $Index
      .$TextWidget window create end -window .$TextWidget.cb$Index -padx 10
 
      if {$Index != $CheckBoxListLastIndex} \
      {
        .$TextWidget insert end "\n"
      }
      
      incr Index
    }
 
    # the bottom buttons
    set ButtonText [::FR_general::ButtonsWithSameWidth [list [msgcat::mc "Ok"] [msgcat::mc "Cancel"]]]
 
    button .$ButtonFrame.okay -text [lindex $ButtonText 0] -state normal -width [lindex $ButtonText 2] -command [list ::tk_cdconverter::FreeDbButtonAction $Window 1]
    button .$ButtonFrame.cancel -text [lindex $ButtonText 1] -state normal -width [lindex $ButtonText 2] -command [list ::tk_cdconverter::FreeDbButtonAction $Window 0]
        
    pack .$ButtonFrame.okay -side left -padx {5 0} -pady 5 -expand yes -fill x
    pack .$ButtonFrame.cancel -side left -padx 5 -pady 5 -expand yes -fill x
  
    bind .$Window <KeyPress-Return> [list .$ButtonFrame.okay invoke]
    bind .$Window <KeyPress-Escape> [list .$ButtonFrame.cancel invoke]
  
    ::FR_emptyBox::Show -window .$Window -returnvalue 0

    vwait ::tk_cdconverter::VAR(FreeDbButtonActionFinished)

    # if the Chancel button was pressed we return
    if {$VAR(FreeDbMultipleValues) eq ""} \
    {
      set VAR(FreeDbTransferFinished) 1
      return
    }
  }
      
  set Genre  [lindex $DataList [expr {$VAR(FreeDbMultipleValues) * 6 + 1}]]
  set DiscId [lindex $DataList [expr {$VAR(FreeDbMultipleValues) * 6 + 3}]]
 
  set Url ""
  append Url $CONFIG(FreeDbAddress) ? [http::formatQuery cmd "cddb read $Genre $DiscId" hello "$env(USER) $CONFIG(Domain) $ABOUT(Name) $ABOUT(Version)" proto "5"]
    
  set CatchReturn [catch {http::geturl $Url -command "::tk_cdconverter::FreeDbParseRead"} CatchContent]
 
  if {$CatchReturn} \
  {
    ::FR_msgWindow::Add -format Error -msg [msgcat::mc "An error occured during connecting to the freedb-database: %s" $CatchContent] 
    set VAR(FreeDbTransferFinished) 2
  }
}


#---the callback function of the choice-window-buttons
proc ::tk_cdconverter::FreeDbButtonAction {Window Action} \
{
  variable VAR
  
  # in case of a cancel command we don't set an index for the 
  if {$Action == 0} \
  {
    set VAR(FreeDbMultipleValues) "" 
  }
     
  set VAR(FreeDbButtonActionFinished) 1 
  focus .
  destroy .$Window
}


#-----this procedure reads the titles from the freedb-database and puts they in a cd-text file-----
proc ::tk_cdconverter::FreeDbParseRead {Token} \
{
  variable VAR;
  
  set Data [http::data $Token]
  
  # 210 is the code for a successful reading, if not: we return
  if {![regexp {^210 ([^\n]*)} $Data]} \
  {
    set ErrorMsg ""
    regexp {(^[0-9]+)([^\n]+)(\n)} $Data ErrorMsg
    
    ::FR_msgWindow::Add -format Error -msg [msgcat::mc "Couldn't get the infos from the freedb-database. The reason was: %s" $ErrorMsg]
   
    http::cleanup $Token
    set VAR(FreeDbTransferFinished) 2
    return
  }
  
  # now we find titles, genre, year, albumtitle and artist
  set Album ""; set AlbumArtist ""; set Year ""; set Genre ""
  set VAR(FreeDbReadResult) ""
  
  set TitleList [regexp -all -inline -- {TITLE([0-9]+)=([^\n]*)} $Data]
  set TitleListLength [llength $TitleList]
  
  regexp {DGENRE=([^\n]*)} $Data All Genre
  regexp {DYEAR=([^\n]*)} $Data All Year
  regexp {DTITLE=([^\n]*) / ([^\n]*)} $Data All AlbumArtist Album
  
  # we check if genre is a known entry 
  if {[lsearch -exact $VAR(GenreListTxt) $Genre] == "-1"} \
  {
    set Genre "Other"
  }
   
  lappend VAR(FreeDbReadResult) $Genre $Year $Album $AlbumArtist
  
  # if there more than one artist on the cd we must read the different artists from the title-entries
  for {set i 1} {$i < $TitleListLength} {incr i 3} \
  {
    set Title ""; set Artist ""
    
    set TitleNumber [lindex $TitleList $i]
    
    set TitleLine [lindex $TitleList [expr {$i + 1}]]
    
    # now we check if the next line the continuation of the last one
    set RunWhileLoop 1
    
    while {$RunWhileLoop} \
    {
      set NextTitleNumberIndex [expr {$i + 3}]
      
      # we don't want to read over the end of the array 
      if {$NextTitleNumberIndex < $TitleListLength} \
      {
        set NextTitleNumber [lindex $TitleList $NextTitleNumberIndex]
      
        # the next line has the same title number -> the next line continues the predecessor line
        if {$TitleNumber == $NextTitleNumber} \
        {
          append TitleLine [lindex $TitleList [expr {$i + 1}]]
          # we look also for further continuations
          incr i 3
        } \
        else \
        {
          # the next line is the next title
          set RunWhileLoop 0
        }
      } \
      else \
      {
        set RunWhileLoop 0
      }
    }
    
    
    if {[string first " / " $TitleLine] != -1} \
    {
      regexp {([^\n]*) / ([^\n]*)} $TitleLine All Artist Title
      lappend VAR(FreeDbReadResult) $Artist $Title
    } \
    else \
    {
      lappend VAR(FreeDbReadResult) $AlbumArtist $TitleLine
    }  
  }

  http::cleanup $Token

  set VAR(FreeDbTransferFinished) 0
}


#---we fill the freedb results into the entry boxes
proc ::tk_cdconverter::FreeDbFillInResults {Index StartTrack EndTrack} \
{
  variable VAR
  
  # we save Genre Year Album AlbumArtist for proc FreeDbFillInCommonFields
  eval lappend VAR(FreeDbCommonFields) [lrange $VAR(FreeDbReadResult) 0 3]
    
  # we don't need Genre Year Album AlbumArtist anymore
  set VAR(FreeDbReadResult) [lreplace $VAR(FreeDbReadResult) 0 3]
  
  for {set i [expr {2 * ($StartTrack - 1)}]} {$i < [expr {2 * $EndTrack}]} {incr i 2} \
  {
    ::tk_cdconverter::TaggerInsertEntryContent .$VAR(TackerTitleFrame).artist$Index [lindex $VAR(FreeDbReadResult) $i]
    ::tk_cdconverter::TaggerInsertEntryContent .$VAR(TackerTitleFrame).title$Index [lindex $VAR(FreeDbReadResult) [expr {$i + 1}]]
    
    incr Index
  }
  
  set VAR(FreeDbReadResult) ""
  
  # we increased Index in the last loop run, so it points already to the next element
  return $Index
}


#---proc fills in the common entry fields
proc ::tk_cdconverter::FreeDbFillInCommonFields {} \
{
  variable VAR; variable CONFIG
  
  set GenreList ""; set YearList ""; set AlbumList ""; set AlbumArtistList ""
  
  foreach "Genre Year Album AlbumArtist" $VAR(FreeDbCommonFields) \
  {
    lappend GenreList $Genre
    lappend YearList $Year
    lappend AlbumList $Album
    lappend AlbumArtistList $AlbumArtist
  }
  
  set VAR(FreeDbCommonFields) ""
  
  set GenreList [::tk_cdconverter::TaggerAllEqual $GenreList]
  set YearList [::tk_cdconverter::TaggerAllEqual $YearList]
  set AlbumList [::tk_cdconverter::TaggerAllEqual $AlbumList]
  set AlbumArtistList [::tk_cdconverter::TaggerAllEqual $AlbumArtistList]

  if {$GenreList ne ""} \
  {
    set CONFIG(Genre) $GenreList
  } \
  else \
  {
    set CONFIG(Genre) "Other" 
  }
  
  if {$YearList ne ""} \
  {
    ::tk_cdconverter::TaggerInsertEntryContent .$VAR(CommonTagsFrame).year $YearList
  } \
  else \
  {
    ::tk_cdconverter::TaggerInsertEntryContent .$VAR(CommonTagsFrame).year
  }
  
  if {$AlbumList ne ""} \
  {
    ::tk_cdconverter::TaggerInsertEntryContent .$VAR(CommonTagsFrame).album $AlbumList
  } \
  else \
  {
    ::tk_cdconverter::TaggerInsertEntryContent .$VAR(CommonTagsFrame).album
  }
  
  if {($AlbumArtistList ne "") && ($AlbumArtistList ne "Various")} \
  {
    ::tk_cdconverter::TaggerInsertEntryContent .$VAR(CommonTagsFrame).artist $AlbumArtistList
  } \
  else \
  {
    ::tk_cdconverter::TaggerInsertEntryContent .$VAR(CommonTagsFrame).artist
  }    
}


#---if wished we delete the *.inf files
proc ::tk_cdconverter::DeleteInfFiles {} \
{
  variable VAR; variable CONFIG
  
  foreach File $VAR(AllFiles) \
  {
    set InfFile [file rootname $File]
    append InfFile ".inf"
    set InfFile [file join $CONFIG(OutputDir) $InfFile]
        
    if {[file exists $InfFile]} \
    {
      catch {file delete $InfFile}
    } 
  }
}


::tk_cdconverter::TaggerMain