#!/bin/sh
#\
exec tclsh "$0" "$@"

# Written by Falk Richter
#
# tk_cdconverter (for a short description please see the About-information) 
#
# This software stands under the GNU GENERAL PUBLIC LICENSE and comes with 
# NO warranty of any kind. Use at your own risk.
# Report bugs or suggestions to: please see contact in the about-box
# ===============================================================================

namespace eval ::tk_cdconverter \
{
  namespace export Main
}


# we load the msgcat package in a global scope  
package require msgcat

# we check if a language is set from the operation system
if {![info exist env(LANG)]} {set env(LANG) "en"}
msgcat::mclocale $env(LANG)
msgcat::mcload [file join [pwd] msgs]


#---the mother of all procedures :o)
proc ::tk_cdconverter::Main {} \
{
  variable CONFIG; variable VAR
 
  ::tk_cdconverter::Startup

  ::tk_cdconverter::Frames 
 
  ::tk_cdconverter::Gui
 
  ::tk_cdconverter::Menu
  
  #::tk_cdconverter::BalloonHelp
  
  ::tk_cdconverter::SearchBackgroundTools
    
  ::tk_cdconverter::GenerateEncoderList
  
  ::tk_cdconverter::Start_cdripper
 
  wm geometry . $CONFIG(Geometry)
  wm deiconify .
  update
}



#---here we load all need packages, set the defaults, read and parse the config file
proc ::tk_cdconverter::Startup {} \
{
  variable VAR; variable ABOUT;
  variable CONFIGFILE; variable DEFAULTS; variable CONFIG;
  global env auto_path tcl_platform
 
  catch {package require Tk}
  wm withdraw .
 
  set VAR(NameSpace) [namespace current]
  set VAR(StartDir) [pwd]
 
  # we check the existens of lib directory   
  set VAR(LibDir) [file join $VAR(StartDir) lib]
 
  if { [file exists $VAR(LibDir)] !="1" } \
  {
    tk_messageBox -message [msgcat::mc "I can't find the program libraries!\n\nPlease install the program again."]\
    -title [msgcat::mc "Where are my libraries?"] -icon error -type ok
    destroy .
    exit
  } else \
  {
    lappend auto_path $VAR(LibDir)
  }
 
  # we load all needed packages
  ::tk_cdconverter::Packageload [list FR_messageBox FR_config FR_general FR_multiFrame FR_msgWindow FR_emptyBox FR_balloonHelp FR_comboBox FR_exec FR_dirBox]
  
  # the informations for the about boxes and the window border
  set InfoFile [file join $VAR(StartDir) ".info.tcl"]
 
  if {[file exists $InfoFile]} \
  {
    source $InfoFile
  } else \
  {
    tk_messageBox -message [msgcat::mc "I can't find the file:\n\n\"%s\"\n\nPlease install the program again." $InfoFile]\
    -title [msgcat::mc "File not found"] -icon error -type ok
    destroy .
    exit
  }
  
  # we load the libaries
  source [file join $VAR(StartDir) tcl tk_cdconverter_lib.tcl] 
  
  # some settings for the toplevel windows
  set VAR(Parent) .
  wm title $VAR(Parent) "$ABOUT(Name) version $ABOUT(Version) from $ABOUT(Date)"
  wm protocol $VAR(Parent) WM_DELETE_WINDOW ::tk_cdconverter::Exit
  
  # the default settings
  set DEFAULTS(BalloonHelp) "1"
  set DEFAULTS(Color) "sandyellow"
  set DEFAULTS(Geometry) "+20+20"
  set DEFAULTS(SourceDir) $VAR(StartDir)
  set DEFAULTS(ReplaceUmlaut) "1"
  set DEFAULTS(Proxy) "0"
  set DEFAULTS(ProxyHost) "dummy.org"
  set DEFAULTS(ProxyPort) "8080"
  set DEFAULTS(Domain) "my.host.com"
  set DEFAULTS(Device) "0,0,0"
  set DEFAULTS(LiveRecording) "0"
  set DEFAULTS(InputDir) $VAR(StartDir)
  set DEFAULTS(OutputDir) $VAR(StartDir)
  set DEFAULTS(InputFormat) "Flac"
  set DEFAULTS(OutputFormat) "Ogg-vorbis"
  set DEFAULTS(DeleteInputFiles) "0"
  set DEFAULTS(BitRate) "128"
  set DEFAULTS(TaggerFormat) "Ogg-vorbis"
  set DEFAULTS(Genre) "Pop"
  set DEFAULTS(UseId3v2) 1
  set DEFAULTS(DeleteInfFiles) "1"
  set DEFAULTS(FreeDbAddress) "http://freedb.freedb.org/~cddb/cddb.cgi"
  set DEFAULTS(RenamerPart1) [msgcat::mc "Titlenumber"]
  set DEFAULTS(RenamerPart2) [msgcat::mc "Artist"]
  set DEFAULTS(RenamerPart3) [msgcat::mc "Title"]
  set DEFAULTS(RenamerPart4) [msgcat::mc "Album"]
  set DEFAULTS(RenamerPart5) [msgcat::mc "Genre"]
  set DEFAULTS(RenamerPart6) [msgcat::mc "Year"]
  set DEFAULTS(RenamerReplaceSpaces) 1
  set DEFAULTS(RenamerSeparator) "::"


  # now we set some os specific defaults
  set VAR(OpenBSD) 0

  switch -glob -- $tcl_platform(os) \
  {
    "OpenBSD" {set VAR(OpenBSD) 1} \
  }
 
  # presetting of some namespace variables
  set VAR(cdripperframe) ""
  set VAR(taggerframe) ""
  set VAR(converterframe) ""
  set VAR(renamerframe) ""
  
  set VAR(ConverterComment) "Created with tk_cdconverter"
  
   
  # we read and parse the configuration file
  set ConfigFileDir [file join $env(HOME) .${ABOUT(Name)}]
 
  if {![file exists $ConfigFileDir]} \
  {
    catch {file mkdir $ConfigFileDir}
  }
 
  set CONFIGFILE [file join $ConfigFileDir .${ABOUT(Name)}_conf.tcl]
 
  ::FR_config::ReadConfig $VAR(NameSpace)

  # we check the config directories and set to the default values if the directory doesn't exist
  foreach Arg {SourceDir InputDir OutputDir} \
  {
    if { ![file isdirectory $CONFIG($Arg)] } \
    {
      set CONFIG($Arg) $DEFAULTS($Arg)
    } 
  }
 
  # we define the "look" (colours)
  set OptionsFile [file join $VAR(StartDir) .tk_options.tcl]
 
  if {[file exists $OptionsFile]} \
  {
    source $OptionsFile
    database_options $VAR(NameSpace)
  }
}


#---we save the configuration and finish the program
proc ::tk_cdconverter::Exit {} \
{
  variable CONFIG; variable VAR
 
  ::FR_config::SaveConfig $VAR(NameSpace)
 
  catch {puts [msgcat::mc "Goodby!"]}
  destroy .
  exit
}


#---the package loader
#---we load the needed package only one time
proc ::tk_cdconverter::Packageload {Packages} \
{
  variable VAR

  foreach Arg $Packages \
  {
    if {![info exists VAR($Arg)]} \
    {
      if { [catch {package require $Arg} output] !="0" } \
      {
        tk_messageBox -message [msgcat::mc "I can't load this part from the program library:\n\n%s\n\nPlease install the program again." $Arg]\
        -title [msgcat::mc "Parts of the program couldn't be loaded"] -icon error -type ok
        destroy .
        exit
      } else \
      {
        set VAR($Arg) 1
      }
    }
  }
}


#---loads an external proc (and read the file if necessary)
proc ::tk_cdconverter::LoadProc {Procedure File} \
{
  variable VAR
 
  set Procedure [string trimleft $Procedure [namespace current]]
  set Procedure [string trimleft $Procedure ::]
  
  if {[info procs $Procedure] eq ""} \
  {
    source [file join $VAR(StartDir) tcl $File]
  }
}


#---We create the frames and set a variable for each frame - 
#---so we can change the order of the frames later
proc ::tk_cdconverter::Frames {} \
{
  variable VAR
 
  frame .frame1 -relief flat
  frame .frame2 -relief flat
  ::FR_multiFrame::Create frame3
  frame .frame4 -relief flat
  frame .frame5 -relief flat

  pack  .frame1 -side top -fill x -padx 5
  pack  .frame2 -side top -fill x -padx 5 -pady 5
  pack  .frame3 -side top -expand yes -fill both -padx 5 -pady 5
  pack  .frame4 -side top -fill x -padx 5 -pady {15 0}
  pack  .frame5 -side top -fill x  -padx 5 -pady {0 5}

  set VAR(menuframe) "frame1"
  set VAR(choiceframe) "frame2"
  set VAR(mainframe) "frame3"
  set VAR(msgwindowlabel) "frame4" 
  set VAR(msgwindow) "frame5"
}


#---This create the menu
proc ::tk_cdconverter::Menu {} \
{
  variable VAR
   
  menubutton .$VAR(menuframe).file -text [msgcat::mc "File"] -menu .$VAR(menuframe).file.choice -underline 0
  menubutton .$VAR(menuframe).pref -text [msgcat::mc "Configuration"] -menu .$VAR(menuframe).pref.choice -underline 0
  menubutton .$VAR(menuframe).about -text [msgcat::mc "About"] -menu .$VAR(menuframe).about.choice -underline 0

  menu .$VAR(menuframe).file.choice -tearoff 0
  .$VAR(menuframe).file.choice add command -label [msgcat::mc "Exit"] -command {::tk_cdconverter::Exit} -accelerator "Ctr+q"
 
  
  menu .$VAR(menuframe).pref.choice -tearoff 0
  .$VAR(menuframe).pref.choice add command -label [msgcat::mc "Options"] -command {::tk_cdconverter::LoadProc ::tk_cdconverter::StartConfig "config.tcl"; ::tk_cdconverter::StartConfig} -accelerator "Ctr+o"
  
  menu .$VAR(menuframe).about.choice -tearoff 0
  .$VAR(menuframe).about.choice add command -label [msgcat::mc "About"] -accelerator "Alt+a" -command [list ::FR_messageBox::Create -type About -nspace $VAR(NameSpace)]
 
  pack .$VAR(menuframe).file -side left
  pack .$VAR(menuframe).pref -side left
  pack .$VAR(menuframe).about -side right
 
  bind . <Control-KeyPress-q> {::tk_msg::Exit}
  bind .$VAR(menuframe).file.choice <Control-KeyPress-q> [list ::FR_general::MenuBinding .$VAR(menuframe).file.choice 0]
  bind .$VAR(menuframe).pref.choice <Control-KeyPress-o> [list ::FR_general::MenuBinding .$VAR(menuframe).pref.choice 0]
  bind .$VAR(menuframe).about.choice <Alt-KeyPress-a> [list ::FR_general::MenuBinding .$VAR(menuframe).about.choice 0]
}


#---we create all widgets
proc ::tk_cdconverter::Gui {} \
{
  variable VAR; variable CONFIG


  # the message window
  ::FR_msgWindow::Create -window .$VAR(msgwindow)

  # the multiframe buttons
  
  # the cdripper button
  button .$VAR(choiceframe).cdripper -text [msgcat::mc "1. Rip a CD\n"] -state normal -command {::tk_cdconverter::Start_cdripper}

  button .$VAR(choiceframe).tagger -text [msgcat::mc "2. Add some informations to\nthe compressed files"] -state normal -command {::tk_cdconverter::Start_tagger}

  button .$VAR(choiceframe).converter -text [msgcat::mc "3. Convert between\n formats"] -state normal -command {::tk_cdconverter::Start_converter}
  
  button .$VAR(choiceframe).renamer -text [msgcat::mc "4. Change the\nfile names"] -state normal -command {::tk_cdconverter::Start_renamer}

  # the Exit button
  button .$VAR(choiceframe).quit -text [msgcat::mc "Exit"] -state normal -command {::tk_cdconverter::Exit}
 
  foreach Arg [list $VAR(choiceframe).cdripper $VAR(choiceframe).tagger $VAR(choiceframe).converter $VAR(choiceframe).renamer $VAR(choiceframe).quit] \
  {
    pack .$Arg -side left -expand yes -fill both
  }
}


#---the start proc for the cdripper
proc ::tk_cdconverter::Start_cdripper {} \
{
  variable VAR
  
  set SubProgramName "cdripper"
    
  if {$VAR(cdripperframe) eq ""} \
  {
    set VAR(cdripperframe) [::FR_multiFrame::NewPage $VAR(mainframe) $SubProgramName [msgcat::mc "CD ripper"]]
    ::tk_cdconverter::LoadProc ::tk_cdconverter::CdripperMain "cdripper.tcl"
    ::FR_multiFrame::Size $::tk_cdconverter::VAR(mainframe) $SubProgramName
  } \
  else \
  {
    ::tk_cdconverter::CdripperClearGui
  }
    
  ::FR_multiFrame::DisplayPage $::tk_cdconverter::VAR(mainframe) $SubProgramName
}


#---the start proc for the tagger
proc ::tk_cdconverter::Start_tagger {} \
{
  variable VAR
  
  set SubProgramName "tagger"
    
  if {$VAR(taggerframe) eq ""} \
  {
    set VAR(taggerframe) [::FR_multiFrame::NewPage $VAR(mainframe) $SubProgramName [msgcat::mc "Tag tool"]]
    ::tk_cdconverter::LoadProc ::tk_cdconverter::TaggerMain "tagger.tcl"
  } \
  else \
  {
    ::tk_cdconverter::TaggerClearGui
  }
    
  ::FR_multiFrame::DisplayPage $::tk_cdconverter::VAR(mainframe) $SubProgramName
}


#---the start proc for the format converter
proc ::tk_cdconverter::Start_converter {} \
{
  variable VAR
  
  set SubProgramName "converter"
    
  if {$VAR(converterframe) eq ""} \
  {
    set VAR(converterframe) [::FR_multiFrame::NewPage $VAR(mainframe) $SubProgramName [msgcat::mc "Format converter"]]
    ::tk_cdconverter::LoadProc ::tk_cdconverter::ConverterMain "converter.tcl"
  } \
  else \
  {
    ::tk_cdconverter::ConverterClearGui
  }
    
  ::FR_multiFrame::DisplayPage $::tk_cdconverter::VAR(mainframe) $SubProgramName
}


#---the start proc for the tagger
proc ::tk_cdconverter::Start_renamer {} \
{
  variable VAR
  
  set SubProgramName "renamer"
    
  if {$VAR(renamerframe) eq ""} \
  {
    set VAR(renamerframe) [::FR_multiFrame::NewPage $VAR(mainframe) $SubProgramName [msgcat::mc "Rename tool"]]
    ::tk_cdconverter::LoadProc ::tk_cdconverter::RenamerMain "renamer.tcl"
  } \
  else \
  {
    ::tk_cdconverter::RenamerClearGui
  }
    
  ::FR_multiFrame::DisplayPage $::tk_cdconverter::VAR(mainframe) $SubProgramName
}


#---we check the optional background tools
proc ::tk_cdconverter::SearchBackgroundTools {} \
{
  variable VAR; variable CONFIG
  global env
  
  set BackgroundTools ""

  lappend BackgroundTools cdda2wav [msgcat::mc "You can't rip CD's."]
  lappend BackgroundTools lame [msgcat::mc "You can't convert wav-files into mp3-files."]
  lappend BackgroundTools mpg321 [msgcat::mc "You can't convert mp3-files into other formats."]
  lappend BackgroundTools id3tag [msgcat::mc "You can't read the extended tags (id3v2) of mp3-files. The program is part of the id3lib-software."]
  lappend BackgroundTools id3info [msgcat::mc "You can't write the extended tags (id3v2) of mp3-files. The program is part of the id3lib-software."]
  lappend BackgroundTools oggenc [msgcat::mc "You can't convert wav-files into ogg-vorbis-files. The program is part of the vorbis-tools-software."]
  lappend BackgroundTools oggdec [msgcat::mc "You can't convert ogg-vorbis-files into other formats. The program is part of the vorbis-tools-software."]
  lappend BackgroundTools vorbiscomment [msgcat::mc "You can't read and write tags of ogg-vorbis-files. The program is part of the vorbis-tools-software."]
  lappend BackgroundTools flac [msgcat::mc "You can't handle flac-files."]
  lappend BackgroundTools metaflac [msgcat::mc "You can't read and write tags of flac-files. The program is part of the flac-software."]

  ::FR_msgWindow::Add -format Heading -msg [msgcat::mc "We are looking for the needed / optional background tools:"]


  for {set i 0} {$i < [llength $BackgroundTools]} {incr i 2} \
  {
    set Tool [lindex $BackgroundTools $i]

    if {[auto_execok $Tool] eq ""} \
    {
      ::FR_msgWindow::Add -format BodyRed -msg [msgcat::mc "I can't find: %s: %s" $Tool [lindex $BackgroundTools [expr {$i+1}]]]
      set VAR($Tool) 0
    } \
    else \
    {
      ::FR_msgWindow::Add -format Body -msg [msgcat::mc "I successful found: %s" $Tool]
      set VAR($Tool) 1
    }
  }
 
  # if no Id3v2 is wished we set the background search back
  if {$CONFIG(UseId3v2) == 0} \
  {
    set VAR(id3info) 0
    set VAR(id3tag) 0
  }
 
  # we check additional the permissions of cdda2wav
  if {$VAR(cdda2wav)} \
  { 
    set Permissions [file attributes [auto_execok cdda2wav] -permissions]
   
    if {(![regexp {[0-9]+4[0-9]+[0-9]+[0-9]+} $Permissions]) && ($env(USER) ne "root")} \
    {
      ::FR_msgWindow::Add -format Info -msg [msgcat::mc "Hint: You are logged in as normal user (very good!) but your cdda2wav havn't set the \"user-id-bit\". This can be a reason for (access-)errors during using cdda2wav. In this case please log in as a superuser and run the ./SetUserId.tcl script in your tk_cdconverter installation directory."]   
    }
  }
}


#---we check the background tools in the later code also: to have a error message why anything dosn't run
proc ::tk_cdconverter::CheckBackgroundTools {Tool} \
{
  variable VAR

  if {$VAR($Tool)} \
  {
    return 1
  } \
  else \
  {
    ::FR_msgWindow::Add -format Error -msg [msgcat::mc "I can't execute this because \"%s\" isn't installed." $Tool]
    return 0
  }
}


#---generate a list with all available encoders
proc ::tk_cdconverter::GenerateEncoderList {} \
{
  variable VAR
  
  # used by the cdripper
  set VAR(EncoderList) ""
  set VAR(EncoderFileExtension) ""

  if {$VAR(cdda2wav)} {lappend VAR(EncoderList) "Wav"; lappend VAR(EncoderFileExtension) "*.wav"}
  if {$VAR(lame)}     {lappend VAR(EncoderList) "Mp3"; lappend VAR(EncoderFileExtension) "*.mp3"}
  if {$VAR(oggenc)}   {lappend VAR(EncoderList) "Ogg-vorbis"; lappend VAR(EncoderFileExtension) "*.ogg"}
  if {$VAR(flac)}     {lappend VAR(EncoderList) "Flac"; lappend VAR(EncoderFileExtension) "*.fla"}
  
  set VAR(DecoderList) "Wav"
  set VAR(DecoderFileExtension) "*.wav"
  
  if {$VAR(mpg321)}   {lappend VAR(DecoderList) "Mp3"; lappend VAR(DecoderFileExtension) "*.mp3"}
  if {$VAR(oggdec)}   {lappend VAR(DecoderList) "Ogg-vorbis"; lappend VAR(DecoderFileExtension) "*.ogg"}
  if {$VAR(flac)}     {lappend VAR(DecoderList) "Flac"; lappend VAR(DecoderFileExtension) "*.fla"}
  
  # used by the tagtool
  set VAR(TagToolList) "Mp3"
  set VAR(TagToolFileExtension) "*.mp3"
  
  if {$VAR(vorbiscomment)}           {lappend VAR(TagToolList) "Ogg-vorbis"; lappend VAR(TagToolFileExtension) "*.ogg"}
  if {$VAR(metaflac)}                {lappend VAR(TagToolList) "Flac"; lappend VAR(TagToolFileExtension) "*.fla"}
}


::tk_cdconverter::Main